/**
 * 
 */
package de.hwrBerlin.ameisenproblem.controller;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.hwrBerlin.ameisenproblem.databeans.Ant;
import de.hwrBerlin.ameisenproblem.databeans.Coordinate;
import de.hwrBerlin.ameisenproblem.databeans.TSPScenario;
import de.hwrBerlin.ameisenproblem.databeans.Transition;
import de.hwrBerlin.ameisenproblem.databeans.TransitionData;
import de.hwrBerlin.ameisenproblem.exceptionhandling.ShowableException;
import de.hwrBerlin.ameisenproblem.view.MainView;

/**
 * @author Nico Smeenk
 * 
 */

public class MainController {
    
    /** View auf der das programm ausgefhrt wird */
    private MainView _view;
    
    /** Liste der Koordinaten */
    private List<Coordinate> _coordinate = new ArrayList<Coordinate>();
    
    /** Map der bergnge zwischen allen Punkten */
    private Map<Transition, TransitionData> _transitions = new HashMap<Transition, TransitionData>();
    
    /** Liste der Ameisen */
    private List<Ant> _ants = new ArrayList<Ant>();
    
    /**
     * Speichert die durchsnittlichen Lngen der Iterationen zum errechnen des gesamtDurchschnittes
     */
    private List<Double> _averageIterationWayLength = new ArrayList<Double>();
    
    /**
     * Speichert das Szenario, wird bei Speicheung genutzt
     */
    private TSPScenario _lastScenario;
    
    /**
     * Speichert die aktuelle zeit
     */
    private long _curTime;
    
    private List<Coordinate> _shortestWay = null;
    
    private boolean _isRunning = false;
    
    

    /**
     * @param pMainView
     */
    public void setView(MainView pMainView) {
        _view = pMainView;
    }
    


    /**
     * ffnet die angegebene Datei
     * 
     * @param pFileOpenName
     */
    public void openTsp(String pFileOpenName) {
        _coordinate = TSPFileParser.parseFile(pFileOpenName, false);
        _view.paintCoordinates(_coordinate);
    }
    


    /**
     * Speichern der Datei under angegebenem Pfad und Namen
     * 
     * @param pFilePath
     */
    public void save(String pFilePath) {
        List<String> header = new ArrayList<String>();
        // Bearbeitung von pFilePath, um Dateinamen fuer Header zu extrahieren.
        header.add("NAME : " + getFileNameWithoutExt(pFilePath));
        TSPFileWriter.writeFile(pFilePath, _coordinate, header, false);
    }
    


    private String getFileNameWithoutExt(String pFilePath) {
        File file = new File(pFilePath);
        String[] fNameList = file.getName().split("\\.");
        String fNameWithoutExt = new String();
        fNameWithoutExt = fNameList[0];
        for (int i = 1; fNameList.length - 2 >= i; i++) {
            fNameWithoutExt += "." + fNameList[i];
        }
        return fNameWithoutExt;
    }
    


    /**
     * Speichern der errechneten Tour
     * 
     * @param pFilePath
     */
    public void saveResult(String pFilePath) {
        System.out.println("Drin1");
        double averageLength = 0;
        for (Double element : _averageIterationWayLength) {
            averageLength += element;
        }
        averageLength = averageLength / _averageIterationWayLength.size();
        
        List<String> header = new ArrayList<String>();
        header.add("NAME : " + getFileNameWithoutExt(pFilePath));
        header.add("DIMENSIONS : " + _shortestWay.size());
        header.add("PARAMETERS");
        header.add("------------------------------");
        header.add("ALPHA : " + _lastScenario.getAlpha());
        header.add("BETA : " + _lastScenario.getBeta());
        header.add("RHO : " + _lastScenario.getRho());
        header.add("TAU0 : " + _lastScenario.getTau0());
        header.add("Q : " + _lastScenario.getQ());
        header.add("------------------------------");
        header.add("TIME NEEDED : " + _curTime + "ms");
        header.add("LENGTH OF AVERAGE ROUTE : " + averageLength);
        header.add("LENGTH OF BEST ROUTE : " + calculateWayLength(_shortestWay));
        header.add("\n");
        TSPFileWriter.writeFile(pFilePath, _shortestWay, header, true);
    }
    


    /**
     * Erstellt ein Zufallsszenario
     */
    public void createRandomSzenario() {
        TSPScenario randomScenario = RandomScenario.getRandomScenario();
        _view.setRandomScenario(randomScenario, true);
        _coordinate = randomScenario.getTspCoordinates();
        _view.paintCoordinates(_coordinate);
        
    }
    
    /**
     * Erstellt nur ZufallsparameterWerte
     */
    public void createRandomValues() {
        TSPScenario randomScenario = RandomScenario.getRandomScenario();
        _view.setRandomScenario(randomScenario, false);
        
    }
    


    /**
     * @param pCoordinate
     */
    public void addCoordinate(Coordinate pCoordinate) {
        _coordinate.add(pCoordinate);
        _view.paintCoordinates(_coordinate);
    }
    


    /**
     * Startet den testfall, initialisiert und geht die iterationen je nach ziel durch
     */
    public void start(TSPScenario pScenario) {
    	_shortestWay = null;
        _isRunning = true;
        // durschnittsliste leeren
        _averageIterationWayLength = new ArrayList<Double>();
        // Speichern der Zeit
        long startTime = new Date().getTime();
        
        // Erstellt die Liste fr die Pheromone und die Lngen zwischen 2
        // Punkten
        initTransitions(pScenario.getTau0());
        
        // Ameisen erstellen
        _ants = new ArrayList<Ant>();
        for (int antIndex = 0; antIndex < pScenario.getNumberAnt(); antIndex++) {
            _ants.add(new Ant());
        }
        
        // Je nach endbedingung unterschiedliche sachen machen
        long time = getTime(startTime);
        _curTime = time;
        if (pScenario.getSkIteration() != 0) {
            for (int iterationIndex = 0; iterationIndex < pScenario.getSkIteration(); iterationIndex++) {
                
                // Iteration ausfhren
                List<Coordinate> shortestIterationWay = runIteration(pScenario);
                
                // Krzesten Weg berechnen
                // erstmaliges setzen des krzesten weges
                if (_shortestWay == null) {
                    _shortestWay = shortestIterationWay;
                }
                // wenn der neue wert aus der iteration kleiner ist als der
                // bislang krzeste, diesen bernehen
                else if (calculateWayLength(_shortestWay) > calculateWayLength(shortestIterationWay)) {
                    _shortestWay = shortestIterationWay;
                }
                
                // Gesamtdurchschnitt berechnen
                double average = 0;
                for (double iterationDurchschnitt : _averageIterationWayLength) {
                    average += iterationDurchschnitt;
                }
                average /= _averageIterationWayLength.size();
                
                // View aktualisieren
                _view.showIteration(Math.round((100 * iterationIndex) / (pScenario.getSkIteration()) + 100 / pScenario.getSkIteration() + 1), _curTime = getTime(startTime),_shortestWay,
                        shortestIterationWay, average, _averageIterationWayLength.get(iterationIndex));
            }
        }
        else if (pScenario.getSkThreshold() != 0) {
            List<Coordinate> bestWay = null;
            double average = 0;
            do {
                // Iteration ausfhren
                bestWay = runIteration(pScenario);
                
                // Krzesten Weg berechnen
                // erstmaliges setzen des krzesten weges
                if (_shortestWay == null) {
                   _shortestWay = bestWay;
                }
                // wenn der neue wert aus der iteration kleiner ist als der
                // bislang krzeste, diesen bernehen
                else if (calculateWayLength(_shortestWay) > calculateWayLength(bestWay)) {
                   _shortestWay = bestWay;
                }
                
                // Gesamtdurchschnitt berechnen
                for (double iterationDurchschnitt : _averageIterationWayLength) {
                    average += iterationDurchschnitt;
                }
                average /= _averageIterationWayLength.size();
                
                // View aktualisieren
                _view.showIteration(50, _curTime = getTime(startTime),_shortestWay, bestWay, average, _averageIterationWayLength.get(_averageIterationWayLength.size() - 1));
                
            }
            while (calculateWayLength(bestWay) > pScenario.getSkThreshold());
            _view.showIteration(100, _curTime = getTime(startTime),_shortestWay, bestWay, average, _averageIterationWayLength.get(_averageIterationWayLength.size() - 1));
        }
        else if (pScenario.getSkBestTourPath() != "") {
            // bestes erreicht
            List<Coordinate> loadedBestWay = getWayFromNameList(TSPFileParser.parseFile(pScenario.getSkBestTourPath(), true));
            System.out.println(TSPFileParser.parseFile(pScenario.getSkBestTourPath(), true));
            double loadedBestWayLength = calculateWayLength(loadedBestWay);
            System.out.println(loadedBestWayLength);
            List<Coordinate> bestWay = null;
            double average = 0;
            do {
                // Iteration ausfhren
                bestWay = runIteration(pScenario);
                
                // Krzesten Weg berechnen
                // erstmaliges setzen des krzesten weges
                if (_shortestWay == null) {
                   _shortestWay = bestWay;
                }
                // wenn der neue wert aus der iteration kleiner ist als der
                // bislang krzeste, diesen bernehen
                else if (calculateWayLength(_shortestWay) > calculateWayLength(bestWay)) {
                   _shortestWay = bestWay;
                }
                
                // Gesamtdurchschnitt berechnen
                for (double iterationDurchschnitt : _averageIterationWayLength) {
                    average += iterationDurchschnitt;
                }
                average /= _averageIterationWayLength.size();
                
                // View aktualisieren
                _view.showIteration(50, _curTime = getTime(startTime),_shortestWay, bestWay, average, _averageIterationWayLength.get(_averageIterationWayLength.size() - 1));
            }
            while (loadedBestWayLength < calculateWayLength(bestWay));
            _view.showIteration(100, _curTime = getTime(startTime),_shortestWay, bestWay, average, _averageIterationWayLength.get(_averageIterationWayLength.size() - 1));
        }
        else {
            // wenn keine sache ausgewhlt wurde exception schmeien
            ShowableException showableException = new ShowableException();
            showableException.setMessage("Kein Endkriterium angegeben!");
            showableException.setTitle("Fehler bei der Eingabe!");
            throw showableException;
        }
        // aktuelles Szenario fuers Ergebnis abspeichern
        _lastScenario = pScenario;
        
        _isRunning = false;
    }
    


    /**
     * @param pParseFile
     * @return
     */
    private List<Coordinate> getWayFromNameList(List<Coordinate> pParseFile) {
        List<Coordinate> returnlist = new ArrayList<Coordinate>();
        for (Coordinate coordinate : pParseFile) {
            for (Coordinate coordinate2 : _coordinate) {
                if (coordinate.getName().equals(coordinate2.getName())) {
                    returnlist.add(coordinate2);
                    break;
                }
            }
        }
        return returnlist;
    }
    


    private long getTime(long startTime) {
        return new Date().getTime() - startTime;
    }
    


    /**
     * @param pScenario
     */
    public void initTransitions(double pTau0) {
        _transitions = new HashMap<Transition, TransitionData>();
        for (int coordinate1Index = 0; coordinate1Index < _coordinate.size(); coordinate1Index++) {
            for (int coordinate2Index = coordinate1Index + 1; coordinate2Index < _coordinate.size(); coordinate2Index++) {
                Transition transition = new Transition(_coordinate.get(coordinate1Index), _coordinate.get(coordinate2Index));
                _transitions.put(transition, new TransitionData(pTau0, calculateLength(transition)));
            }
        }
        // for (Coordinate coordinate1 : _coordinate) {
        // for (Coordinate coordinate2 : _coordinate) {
        // Transition transition = new Transition(coordinate1, coordinate2);
        // _transitions.put(transition, new TransitionData(pTau0,
        // calculateLength(transition)));
        // }
        // }
        
    }
    


    /**
     * berechnet die Lnge einer strecke und gibt diese zurck
     * 
     * @param pTransition
     * @return
     */
    public static double calculateLength(Transition pTransition) {
        // pytagoras a + b = c
        double a = pTransition.getCoordinate1().getxValue() - pTransition.getCoordinate2().getxValue();
        double b = pTransition.getCoordinate1().getyValue() - pTransition.getCoordinate2().getyValue();
        double c = Math.sqrt(a * a + b * b);
        
        return c;
    }
    


    /**
     * berechnet die Lnge eines weges
     * 
     * @param pWay
     * @return
     */
    public static double calculateWayLength(List<Coordinate> pWay) {
        double length = 0;
        // alle wege zusammenrechnen
        for (int coordinateIndex = 0; coordinateIndex < pWay.size() - 1; coordinateIndex++) {
            length += calculateLength(new Transition(pWay.get(coordinateIndex), pWay.get(coordinateIndex + 1)));
        }
        // zuletzt weg vom ende zum startpunkt mit einberechnen
        length += calculateLength(new Transition(pWay.get(pWay.size() - 1), pWay.get(0)));
        return length;
    }
    


    /**
     * durchluft eine Iteration
     * 
     * @param pScenario
     * 
     * @return gibt die krzeste Strecke der Iteration zurck
     */
    private List<Coordinate> runIteration(TSPScenario pScenario) {
        // alle Ameisen laufen lassen
        for (Ant ant : _ants) {
            runAnt(pScenario.getAlpha(), pScenario.getBeta(), ant);
            // pheromoneUpdate(pScenario.getRho(), pScenario.getQ());
        }
        
        // dannach Pheromone aktualisieren
        pheromoneUpdate(pScenario.getRho(), pScenario.getQ());
        
        // Krzesten und durchschnittlichen Weg ermitteln
        List<Coordinate> bestWay = null;
        double averageWay = 0;
        for (Ant ant : _ants) {
            averageWay += calculateWayLength(ant.getWay());
            if (bestWay == null) {
                bestWay = ant.getWay();
            }
            else if (calculateWayLength(bestWay) > calculateWayLength(ant.getWay())) {
                bestWay = ant.getWay();
            }
        }
        averageWay /= _ants.size();
        
        // durchschnittlichen Weg in das listenatribut eintragen
        _averageIterationWayLength.add(averageWay);
        
        // Krzesten Weg zurckgeben
        return bestWay;
    }
    


    /**
     * Lsst eine Ameise laufen
     * 
     * @param pAnt
     */
    public void runAnt(double pAlpha, double pBeta, Ant pAnt) {
        // Weg muss gelscht werden
        pAnt.removeWay();
        
        // der Ameise muss ein neuer zuflliger Startwert gegeben werden
        pAnt.addCoordinateToWay(_coordinate.get((int) (Math.random() * _coordinate.size())));
        
        // Ameisenweg berechnen
        for (int i = 0; i < _coordinate.size() - 1; i++) {
            Map<Coordinate, Double> probabilities = getTransitionProbability(pAlpha, pBeta, pAnt);
            
            // Summe berechnen
            double sum = 0;
            for (Coordinate coordinate : probabilities.keySet()) {
                sum += probabilities.get(coordinate);
            }
            
            double random = Math.random() * sum;
            
            double isAtPosition = 0;
            for (Coordinate coordinate : probabilities.keySet()) {
                isAtPosition += probabilities.get(coordinate);
                if (random < isAtPosition) {
                    pAnt.addCoordinateToWay(coordinate);
                    break;
                }
            }
        }
        if (pAnt.getWay().size() != _coordinate.size()) {
            ShowableException showableException = new ShowableException();
            showableException.setTitle("Fehler des Algoritmus");
            showableException.setMessage("Durch die Angabe der Parameter ist ein Punkt unerreichbar geworden!");
            throw showableException;
        }
    }
    


    /**
     * gibt die Wahrscheinloichkeiten fr die anderen Stdte
     * 
     * @param pAlpha
     * @param pBeta
     * @param pTransition
     * @param pAnt
     * @return
     */
    public Map<Coordinate, Double> getTransitionProbability(double pAlpha, double pBeta, Ant pAnt) {
        Map<Coordinate, Double> returnMap = new HashMap<Coordinate, Double>();
        
        for (Coordinate coordinate : getCanGoToCoordinate(pAnt)) {
            returnMap.put(coordinate, getVektorProbability(pAlpha, pBeta, new Transition(pAnt.getLastCity(), coordinate)));
        }
        
        return returnMap;
    }
    


    /**
     * Gibt die berechnete Wahrscheinlichkeit fr den angegeben Vektor
     * 
     * @param pAlpha
     * @param pBeta
     * @param pTransition
     * @return
     */
    public double getVektorProbability(double pAlpha, double pBeta, Transition pTransition) {
        double a = Math.pow(_transitions.get(pTransition).getPheromone(), pAlpha);
        double b = Math.pow(_transitions.get(pTransition).getLength(), pBeta);
        return a * (1 / b);
    }
    


    /**
     * Gibt die Felder zurck die eine Ameise noch nicht besucht hat.
     * 
     * @param pAnt
     * @return
     */
    public List<Coordinate> getCanGoToCoordinate(Ant pAnt) {
        List<Coordinate> possibleCities = new ArrayList<Coordinate>();
        
        for (Coordinate coordinate : _coordinate) {
            if (!pAnt.getWay().contains(coordinate)) {
                possibleCities.add(coordinate);
            }
        }
        
        return possibleCities;
    }
    


    /**
     * @param pTransition
     * @return
     */
    public TransitionData getTransitionData(Transition pTransition) {
        return _transitions.get(pTransition);
    }
    


    /*
     * Diese Methode kann nach einer speziellen Formel und den Parametern aus dem GUI (Q) die Liste der Koordinaten und den zugehrigen Pheromonwerten aktualisieren. Dabei werden die aktuellen Touren
     * der Ameisen benutzt.
     */
    private void pheromoneUpdate(double rho, double q) {
        // fr jede Ameise den Weg updaten
        for (Ant ant : _ants) {
            double antWayLength = calculateWayLength(ant.getWay());
            // Update jedes Weges den die Ameise gelaufen ist
            for (int coordinateIndex = 0; coordinateIndex < ant.getWay().size() - 1; coordinateIndex++) {
                Transition transition1 = new Transition(ant.getWay().get(coordinateIndex), ant.getWay().get(coordinateIndex + 1));
                TransitionData data = _transitions.get(transition1);
                // aktualisieren
                double deltaPheromon = q / antWayLength;
                data.setPheromone(data.getPheromone() + deltaPheromon);
            }
        }
        // Verdunstung
        for (Transition transition : _transitions.keySet()) {
            double oldPheromon = _transitions.get(transition).getPheromone();
            _transitions.get(transition).setPheromone((1 - rho) * oldPheromon);
        }
        
    }
    


    /**
     * gibt zurck ob die Koordinaten gesetzt sine
     * 
     * @return
     */
    public boolean coordinatesSet() {
        if (_coordinate.size() > 0) {
            return true;
        }
        return false;
    }
    


    /**
     * @return
     */
    public boolean isRunning() {
        return _isRunning;
    }
    


    public void setIsRunning(boolean pRunning) {
        _isRunning = pRunning;
    }



    /**
     * gibt zurck ob der Name bereits verwendet wird
     * @param pText
     * @return
     */
    public boolean isCityNameInUse(String pText) {
        for (Coordinate coordinate : _coordinate) {
            if (coordinate.getName().equals(pText)){
                return true;
            }
        }
        return false;
    }



    /**
     * @return
     */
    public List<Coordinate> getCoordinates() {
        return _coordinate;
    }

    /**
     * @return the shortestWay
     */
    public List<Coordinate> getShortestWay() {
        return _shortestWay;
    }
}
