package de.hwrBerlin.ameisenproblem.controller;
/**
 * Diese Klasse erstellt ein zuflliges Scenario.
 * Alle Werte sind zufllig, jedoch wird festgelegt,
 * dass das StopKriterium als Anzahl von Iteration benutzt wird.
 * (Es kann nicht garantiert werden, dass eine ideale Tour bekannt ist.)
 */


import java.util.LinkedList;
import java.util.Random;

import de.hwrBerlin.ameisenproblem.databeans.Coordinate;
import de.hwrBerlin.ameisenproblem.databeans.TSPScenario;

public class RandomScenario {
	
	public static final int MIN_NUMBER_OF_CITIES = 50;  
	public static final int MIN_NUMBER_OF_ANTS = 20;
	public static final int MIN_NUMBER_OF_ITERATIONS = 20;
	
	private static Random random;
	
	public static TSPScenario getRandomScenario() {
		TSPScenario randomScenario = new TSPScenario();
		
		random = new Random();
		
		//Flle die Werte zufllig.
		//Double-Werte
		randomScenario.setAlpha(random.nextDouble());
		randomScenario.setBeta(random.nextDouble());
		randomScenario.setQ(getMinmalRandomInt(0, 10) + random.nextDouble() + 2);
		randomScenario.setRho(random.nextDouble());
		randomScenario.setTau0(getMinmalRandomInt(0, 10) + random.nextDouble() + 1);
		
		
		//Integer Werte, wobei eine bestimmte Mindestgre eingehalten werden muss.
		
		randomScenario.setSkIteration(getMinmalRandomInt(MIN_NUMBER_OF_ITERATIONS, 100));
		randomScenario.setNumberAnt(getMinmalRandomInt(MIN_NUMBER_OF_ANTS, 200));
		randomScenario.setNumberCities(getMinmalRandomInt(MIN_NUMBER_OF_CITIES, 600));
		
		//Es mssen zufllige Koordinaten erstellt werden!
		randomScenario.setTspCoordinates(getRandomCoordinateList(randomScenario.getNumberCities()));
		
		
		return randomScenario;
	}
	
	/**
	 * Diese Methode generiert einen zuflligen int-Wert
	 * der grer ist als wie der bergebene Parameter.
	 * @param minimal Der kleinste erlaubte Integer-Wert.
	 * @return Gibt einen zuflligen integer Wert zurck.
	 */
	private static int getMinmalRandomInt(int minimal, int maximal) {
	    //Das darf nicht passieren
	    if(minimal >= maximal) {
	        return 1;
	    }
		int tempint;
		
		while((tempint = random.nextInt(maximal)) <= minimal) {
			//Do Nothing
		}
		
		return tempint;
	}
	
	/**
	 * Erstellt zufllig eine Liste von Coordinaten
	 * @param numberOfCities Wie viele Koordinaten sollen generiert werden?
	 * 						Sollte gleich gesetzt mit numberOfCities in Scenario sein!
	 * @return	Gibt eine zuf
	 */
	private static LinkedList<Coordinate> getRandomCoordinateList(int numberOfCities) {
		LinkedList<Coordinate> randomCoordinateList = new LinkedList<Coordinate>();
		
		for (int i = 1; i <= numberOfCities; i++) {
			Coordinate tempCoordinate = new Coordinate();
			tempCoordinate.setName(""+i);
			tempCoordinate.setxValue(random.nextDouble() * random.nextInt(10000));
			tempCoordinate.setyValue(random.nextDouble() * random.nextInt(10000));
			randomCoordinateList.add(tempCoordinate);
		}
		
		return randomCoordinateList;
	}

}
