package de.hwrBerlin.ameisenproblem.controller;

import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.List;

import de.hwrBerlin.ameisenproblem.databeans.Coordinate;
import de.hwrBerlin.ameisenproblem.exceptionhandling.ShowableException;
import de.hwrBerlin.ameisenproblem.view.Constants;

/**
 * 
 * @author Benjamin, Jonathan Methoden zum Abspeichern von Koordinaten und des krzesten gefundenen Weges
 */
public class TSPFileWriter {
    
    /**
     * Erzeugt eine ".tsp" Datei, die die Koordinaten des gerade erzeugten Graphens.
     * 
     * @param fileName
     *            Datenpfad der abzuspechernden Datei (mit .tsp)
     * @param coordinates
     *            LikedList der Koordinaten
     * @param header
     *            LinkedList mit allen Zeilen des Headers (Ausgenommen HeaderEnd)
     * @param isTour
     *            Wenn es sich um eine .tsp Datei handelt, dann wird False mitgegeben, fr eine Tour wird True erwartet.
     * @return true, falls erfolgreich abgespeichert wurde false, sonst.
     */
    public static Boolean writeFile(String fileName, List<Coordinate> coordinates, List<String> header, boolean isTour) {
        // Prfe ob die bergebenen Parameter nicht leer sind
        if (fileName == null || fileName.trim().equals("")) {
            System.out.println("TSPFileWriter" + ": Es wurde kein Dateiname angegeben");
            return false;
        }
        if (header == null || header.size() == 0) {
            System.out.println("TSPFileWriter" + ": Es wurde kein Header / leerer Header mitgegeben");
            return false;
        }
        if (coordinates == null || coordinates.size() == 0) {
            System.out.println("TSPFileWriter" + ": Es wurden keine Koordinaten / leere Koordinaten mitgegeben");
            return false;
        }
        
        // Schreibe Datei
        PrintWriter pw;
        
        try {
            pw = new PrintWriter(new FileWriter(fileName));
            
            // bernehme den Header und schreibe ihn in die Datei
            for (String headerLine : header) {
                if (!headerLine.trim().equals("") && headerLine != null) {
                    pw.println(headerLine);
                }
            }
            //Zur sicherheit Schreibe den EndTag selbst in den Header (Tour oder TSP)
            if(isTour) {
            	pw.println(Constants.TOUR_HEADER_END);
            }
            else {
            	pw.println(Constants.TSP_HEADER_END);
            }
            
            
            
            // Falls es eine .tsp Datei sein soll
            if (!isTour) {
                // bernehme den Name und die Koordinaten und schreibe sie in die Datei
                for (Coordinate coordinateLine : coordinates) {
                    String tempCoordinateString;
                    tempCoordinateString = coordinateLine.getName() + " " + coordinateLine.getxValue() + " " + coordinateLine.getyValue();
                    pw.println(tempCoordinateString);
                }
            }
            // Ansonsten soll es eine .tour Datei sein.
            else {
                for (Coordinate coordinateLine : coordinates) {
                    pw.println(coordinateLine.getName());
                }
            }
            pw.println("-1");
            pw.println(Constants.FILE_END);
            pw.close();
            return true;
        }
        catch (Exception e) {

            System.out.println("TSPFileWriter" + ": Fehler beim Schreiben der Datei: " + fileName);
          //Werfe eine Exception und benachrichtige den Benutzer
            ShowableException exception = new ShowableException();
            exception.setMessage("Fehler beim Schreiben der Datei: " + fileName +" . Bitte dem Support melden!");
            e.printStackTrace();
            throw exception;
        }
    }
    
}
