/**
 * 
 */
package de.hwrBerlin.ameisenproblem.view;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Text;

import de.hwrBerlin.ameisenproblem.controller.MainController;
import de.hwrBerlin.ameisenproblem.databeans.TSPScenario;
import de.hwrBerlin.ameisenproblem.exceptionhandling.ShowableException;

/**
 * 
 * @author Christian Gebauer, Sebastian Groe
 * 
 *         Anzeige der Konfigurationsmglichkeiten fr das TSP
 * 
 */
public class ConfigurationView extends Composite {
    
    /** Controller fr die Logik */
    private MainController _controller;
    
    // Textwerte der EIngabeansicht
    private Text _alphaText;
    private Text _betaText;
    private Text _rhoText;
    private Text _tau0Text;
    private Text _qText;
    private Text _skIterationText;
    private Text _skThresholdText;
    private Text _skSolutionBrowseText;
    private Text _numberAntText;
    
    // IterationsButton muss von aussen nderbar sein
    private Button _skIterationButton;
    private Button _skSolutionBrowseButton;
    private Button _skSolutionButton;
    private Button _skThresholdButton;
    
    private Color _red = new Color(getDisplay(), Constants.RED_COLOR);
    private Color _white = new Color(getDisplay(), Constants.WHITE_COLOR);
    private Color _gray = new Color(getDisplay(), 240, 240, 240);
    
    private Button _startButton;
    
    /** Logikthread */
    private Thread thread;
    
    

    /**
     * @param pParent
     * @param pStyle
     */
    public ConfigurationView(final Composite pParent, int pStyle, MainController pMainController) {
        super(pParent, pStyle);
        
        _controller = pMainController;
        
        setLayout(new RowLayout(SWT.VERTICAL));
        
        RowLayout rowLayout = new RowLayout();
        rowLayout.center = true;
        
        RowData defaultParamRowData = new RowData(25, 15);
        // GridData defaultStopCriterionGridData = new GridData(16, 16);
        RowData defaultNumberAntRowData = new RowData(50, 15);
        
        Label resultTitleLabel = new Label(this, SWT.NONE);
        resultTitleLabel.setText("Eingabe");
        resultTitleLabel.setFont(new Font(pParent.getDisplay(), "Arial", 16, SWT.BOLD));
        
        // Eingabefelder der Parameter
        Label configurationLabel = new Label(this, SWT.NONE);
        configurationLabel.setText("Konfiguration der Parameter:");
        configurationLabel.setFont(new Font(pParent.getDisplay(), "Arial", 10, SWT.BOLD));
        
        Composite paramComposite = new Composite(this, SWT.NONE);
        paramComposite.setLayout(rowLayout);
        
        Label alphaLabel = new Label(paramComposite, SWT.NONE);
        alphaLabel.setText(Constants.PHEROMON_PARAMETER_VARIABLE);
        alphaLabel.setToolTipText(Constants.PHEROMON_PARAMETER_TOOLTIP);
        
        _alphaText = new Text(paramComposite, SWT.BORDER);
        _alphaText.setLayoutData(defaultParamRowData);
        _alphaText.setData(false);
        _alphaText.setBackground(_red);
        
        Label betaLabel = new Label(paramComposite, SWT.NONE);
        betaLabel.setText(Constants.HEURISTISCHER_PARAMETER_VARIABLE);
        betaLabel.setToolTipText(Constants.HEURISTISCHER_PARAMETER_TOOLTIP);
        
        _betaText = new Text(paramComposite, SWT.BORDER);
        _betaText.setLayoutData(defaultParamRowData);
        _betaText.setData(false);
        _betaText.setBackground(_red);
        
        Label rhoLabel = new Label(paramComposite, SWT.NONE);
        rhoLabel.setText(Constants.VERDUNSTUNGSFAKTOR_VARIABLE);
        rhoLabel.setToolTipText(Constants.VERDUNSTUNGSFAKTOR_TOOLTIP);
        
        _rhoText = new Text(paramComposite, SWT.BORDER);
        _rhoText.setLayoutData(defaultParamRowData);
        _rhoText.setData(false);
        _rhoText.setBackground(_red);
        
        Label tau0Label = new Label(paramComposite, SWT.NONE);
        tau0Label.setText(Constants.INITIALER_PHEROMON_WERT_VARIABLE);
        tau0Label.setToolTipText(Constants.INITIALER_PHEROMON_TOOLTIP);
        
        _tau0Text = new Text(paramComposite, SWT.BORDER);
        _tau0Text.setLayoutData(defaultParamRowData);
        _tau0Text.setData(false);
        _tau0Text.setBackground(_red);
        
        Label qLabel = new Label(paramComposite, SWT.NONE);
        qLabel.setText(Constants.PHEROMON_UPDATE_VARIABLE);
        qLabel.setToolTipText(Constants.PHEROMON_UPDATE_TOOLTIP);
        
        _qText = new Text(paramComposite, SWT.BORDER);
        _qText.setLayoutData(defaultParamRowData);
        _qText.setData(false);
        _qText.setBackground(_red);
        
        // Beschreibungen der Parameter
        Label alphaDescriptionLabel = new Label(this, SWT.NONE);
        alphaDescriptionLabel.setText(Constants.PHEROMON_PARAMETER_VARIABLE_DESCRIPTION);
        alphaDescriptionLabel.setToolTipText(Constants.PHEROMON_PARAMETER_TOOLTIP);
        
        Label betaDescriptionLabel = new Label(this, SWT.NONE);
        betaDescriptionLabel.setText(Constants.HEURISTISCHER_PARAMETER_VARIABLE_DESCRIPTION);
        betaDescriptionLabel.setToolTipText(Constants.HEURISTISCHER_PARAMETER_TOOLTIP);
        
        Label rhoDescriptionLabel = new Label(this, SWT.NONE);
        rhoDescriptionLabel.setText(Constants.VERDUNSTUNGSFAKTOR_VARIABLE_DESCRIPTION);
        rhoDescriptionLabel.setToolTipText(Constants.VERDUNSTUNGSFAKTOR_TOOLTIP);
        
        Label tauDescriptionLabel = new Label(this, SWT.NONE);
        tauDescriptionLabel.setText(Constants.INITIALER_PHEROMON_WERT_VARIABLE_DESCRIPTION);
        tauDescriptionLabel.setToolTipText(Constants.INITIALER_PHEROMON_TOOLTIP);
        
        Label qDescriptionLabel = new Label(this, SWT.NONE);
        qDescriptionLabel.setText(Constants.PHEROMON_UPDATE_VARIABLE_DESCRIPTION);
        qDescriptionLabel.setToolTipText(Constants.PHEROMON_UPDATE_TOOLTIP);
        
        Label lineLabel = new Label(this, SWT.SEPARATOR | SWT.HORIZONTAL | SWT.CENTER);
        lineLabel.setLayoutData(new RowData(320, 20));
        
        // Stoppkriterien
        Label stopCriterionLabel = new Label(this, SWT.NONE);
        stopCriterionLabel.setText("Stoppkriterium:");
        stopCriterionLabel.setFont(new Font(pParent.getDisplay(), "Arial", 10, SWT.BOLD));
        
        Composite stopCriterionComposite = new Composite(this, SWT.NONE);
        stopCriterionComposite.setLayout(new GridLayout(3, false));
        
        _skIterationButton = new Button(stopCriterionComposite, SWT.RADIO);
        _skIterationButton.setText("Anzahl der Iterationen");
        
        GridData span2GridData = new GridData();
        span2GridData.horizontalSpan = 2;
        
        _skIterationText = new Text(stopCriterionComposite, SWT.BORDER);
        _skIterationText.setLayoutData(span2GridData);
        _skIterationText.setEnabled(false);
        _skIterationText.setData(false);
        _skIterationText.setBackground(_red);
        
        _skThresholdButton = new Button(stopCriterionComposite, SWT.RADIO);
        _skThresholdButton.setText("Schwellwert erreicht");
        
        _skThresholdText = new Text(stopCriterionComposite, SWT.BORDER);
        _skThresholdText.setLayoutData(span2GridData);
        _skThresholdText.setEnabled(false);
        _skThresholdText.setData(false);
        _skThresholdText.setBackground(_red);
        
        _skSolutionButton = new Button(stopCriterionComposite, SWT.RADIO);
        _skSolutionButton.setText("Lsung gefunden");
        
        _skSolutionBrowseText = new Text(stopCriterionComposite, SWT.BORDER | SWT.READ_ONLY);
        _skSolutionBrowseText.setLayoutData(new GridData(120, 16));
        _skSolutionBrowseText.setEnabled(false);
        _skSolutionBrowseText.setData(false);
        _skSolutionBrowseText.setBackground(_red);
        
        _skSolutionBrowseButton = new Button(stopCriterionComposite, SWT.PUSH);
        _skSolutionBrowseButton.setToolTipText("Browse");
        _skSolutionBrowseButton.setImage(new Image(getDisplay(), Constants.BROWSE_IMAGE));
        _skSolutionBrowseButton.setEnabled(false);
        
        // Label emptyLabel = new Label(this, SWT.NONE);
        // emptyLabel.setText("");
        
        // Anzahl der Ameisen
        Composite numberAntComposite = new Composite(this, SWT.NONE);
        numberAntComposite.setLayout(rowLayout);
        numberAntComposite.setLayoutData(new RowData(200, 35));
        // numberAntComposite.setBackground(new Color(getDisplay(), 255,0,0));
        
        Label numberAntLabel = new Label(numberAntComposite, SWT.NONE);
        numberAntLabel.setText("Anzahl Ameisen: ");
        
        _numberAntText = new Text(numberAntComposite, SWT.BORDER);
        _numberAntText.setLayoutData(defaultNumberAntRowData);
        _numberAntText.setBackground(_red);
        _numberAntText.setData(false);
        
        // Label emptyLabel2 = new Label(this, SWT.NONE);
        // emptyLabel2.setText("");
        
        // StartButton
        _startButton = new Button(this, SWT.PUSH);
        _startButton.setText("START");
        _startButton.setFont(new Font(pParent.getDisplay(), "Arial", 16, SWT.BOLD));
        _startButton.setEnabled(false);
        
        // ============================================================================================
        // Events
        // ============================================================================================
        
        // nur Zahlen als eingabe akzeptieren
        
        ModifyListener modifyListenerSmaller1 = new ModifyListener() {
            
            @Override
            public void modifyText(ModifyEvent pE) {
                Text field = (Text) pE.widget;
                String text = field.getText();
                if (text.matches("(|0|0\\.[0]*)")) {
                    field.setData(false);
                    field.setBackground(_red);
                }
                else {
                    field.setData(true);
                    field.setBackground(_white);
                    if (text.matches("(0\\.[0-9]*|1|1\\.[0]*)")) {
                    }
                    else {
                        field.setData(false);
                        field.setBackground(_red);
                    }
                }
                refreshStartButton();
            }
        };
        ModifyListener modifyListenerBigger0 = new ModifyListener() {
            
            @Override
            public void modifyText(ModifyEvent pE) {
                Text field = (Text) pE.widget;
                String text = field.getText();
                if (text.matches("(|0|0\\.[0]*)")) {
                    field.setData(false);
                    field.setBackground(_red);
                }
                else {
                    field.setData(true);
                    field.setBackground(_white);
                    if (text.matches("([0-9]*\\.[0-9]*|[0-9]*)")) {
                    }
                    else {
                        field.setData(false);
                        field.setBackground(_red);
                    }
                }
                refreshStartButton();
            }
        };
        
        _alphaText.addModifyListener(modifyListenerSmaller1);
        _betaText.addModifyListener(modifyListenerSmaller1);
        _rhoText.addModifyListener(modifyListenerSmaller1);
        
        _tau0Text.addModifyListener(modifyListenerBigger0);
        _qText.addModifyListener(modifyListenerBigger0);
        
        _numberAntText.addModifyListener(new ModifyListener() {
            
            @Override
            public void modifyText(ModifyEvent pE) {
                Text field = (Text) pE.widget;
                String text = field.getText();
                if (text.matches("[0]*")) {
                    field.setData(false);
                    field.setBackground(_red);
                }
                else {
                    field.setData(true);
                    field.setBackground(_white);
                    if (text.matches("[0-9]*")) {
                    }
                    else {
                        field.setData(false);
                        field.setBackground(_red);
                    }
                }
                refreshStartButton();
            }
        });
        
        _skIterationText.addModifyListener(new ModifyListener() {
            
            @Override
            public void modifyText(ModifyEvent pE) {
                Text field = (Text) pE.widget;
                String text = field.getText();
                if (text.matches("[0-9][0-9]*")) {
                    field.setData(true);
                    field.setBackground(_white);
                }
                else {
                    field.setData(false);
                    field.setBackground(_red);
                }
                refreshStartButton();
            }
        });
        _skThresholdText.addModifyListener(modifyListenerBigger0);
        _skSolutionBrowseText.addModifyListener(new ModifyListener() {
            
            @Override
            public void modifyText(ModifyEvent pE) {
                Text field = (Text) pE.widget;
                String text = field.getText();
                if (text != "") {
                    field.setData(true);
                    field.setBackground(_white);
                }
                else {
                    field.setData(false);
                    field.setBackground(_red);
                }
                refreshStartButton();
            }
        });
        
        // Radiobuttonauswahl der Stopkriterien
        _skIterationButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _skIterationText.setEnabled(true);
                _skIterationText.setText(_skIterationText.getText());
                _skThresholdText.setEnabled(false);
                _skThresholdText.setBackground(_gray);
                _skSolutionBrowseButton.setEnabled(false);
                _skSolutionBrowseText.setEnabled(false);
                _skSolutionBrowseText.setBackground(_gray);
            }
        });
        
        _skThresholdButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _skIterationText.setEnabled(false);
                _skIterationText.setBackground(_gray);
                _skThresholdText.setEnabled(true);
                _skThresholdText.setText(_skThresholdText.getText());
                _skSolutionBrowseButton.setEnabled(false);
                _skSolutionBrowseText.setEnabled(false);
                _skSolutionBrowseText.setBackground(_gray);
            }
        });
        
        _skSolutionButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _skIterationText.setEnabled(false);
                _skIterationText.setBackground(_gray);
                _skThresholdText.setEnabled(false);
                _skThresholdText.setBackground(_gray);
                _skSolutionBrowseButton.setEnabled(true);
                _skSolutionBrowseText.setEnabled(true);
                _skSolutionBrowseText.setText(_skSolutionBrowseText.getText());
            }
        });
        
        _skSolutionBrowseButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                
                FileDialog fileOpenDialog = new FileDialog(pParent.getShell(), SWT.OPEN);
                fileOpenDialog.setFilterExtensions(Constants.TOUR_FILE_EXTENSION);
                String fileName = fileOpenDialog.open();
                if (fileName != null) {
                    _skSolutionBrowseText.setText(fileName);
                    _skSolutionBrowseText.setData(true);
                }
            }
        });
        
        // Startbutton
        _startButton.addSelectionListener(new SelectionAdapter() {
            
            @Override
            public void widgetSelected(SelectionEvent e) {
                if (thread == null) {
                    
                    // werte beprfen und zuweisen
                    
                    final TSPScenario scenario = new TSPScenario();
                    scenario.setAlpha(Double.parseDouble(_alphaText.getText()));
                    scenario.setBeta(Double.parseDouble(_betaText.getText()));
                    scenario.setRho(Double.parseDouble(_rhoText.getText()));
                    scenario.setTau0(Double.parseDouble(_tau0Text.getText()));
                    scenario.setQ(Double.parseDouble(_qText.getText()));
                    scenario.setNumberAnt(Integer.parseInt(_numberAntText.getText()));
                    // Stoppkriterium setzen
                    if (_skIterationButton.getSelection()) {
                        scenario.setSkIteration(Integer.parseInt(_skIterationText.getText()));
                    }
                    else if (_skSolutionButton.getSelection()) {
                        scenario.setSkBestTourPath(_skSolutionBrowseText.getText());
                    }
                    else if (_skThresholdButton.getSelection()) {
                        scenario.setSkThreshold(Integer.parseInt(_skThresholdText.getText()));
                    }
                    
                    setStopButton();
                    
                    // Logik in neuem Thred starten
                    thread = new Thread() {
                        /*
                         * (non-Javadoc)
                         * 
                         * @see java.lang.Thread#run()
                         */
                        @Override
                        public void run() {
                            try {
                                _controller.start(scenario);
                            }
                            catch (final ShowableException e) {
                                getDisplay().asyncExec(new Runnable() {
                                    @Override
                                    public void run() {
                                        MessageBox messageBox = new MessageBox(getShell(), SWT.ICON_ERROR);
                                        messageBox.setText(e.getTitle());
                                        messageBox.setMessage(e.getMessage());
                                        messageBox.open();
                                    }
                                });
                                _controller.setIsRunning(false);
                                setStartButton();
                            }
                            if (thread != null) {
                                setStartButton();
                            }
                        }
                        
                    };
                    thread.start();
                }
                else {
                    setStartButton();
                }
            }
            


            private void setStartButton() {
                getDisplay().asyncExec(new Runnable() {
                    @SuppressWarnings("deprecation")
                    @Override
                    public void run() {
                        if (thread != null) {
                            thread.stop();
                            thread = null;
                        }
                        _startButton.setText("START");
                        
                        _alphaText.setEnabled(true);
                        _betaText.setEnabled(true);
                        _rhoText.setEnabled(true);
                        _tau0Text.setEnabled(true);
                        _qText.setEnabled(true);
                        _numberAntText.setEnabled(true);
                        _skIterationButton.setEnabled(true);
                        _skIterationText.setEnabled(true);
                        _skThresholdButton.setEnabled(true);
                        _skThresholdText.setEnabled(true);
                        _skSolutionButton.setEnabled(true);
                        _skSolutionBrowseButton.setEnabled(true);
                        _skSolutionBrowseText.setEnabled(true);
                        
                        _controller.setIsRunning(false);
                    }
                });
            }
            


            private void setStopButton() {
                _startButton.setText("STOP");
                
                _alphaText.setEnabled(false);
                _betaText.setEnabled(false);
                _rhoText.setEnabled(false);
                _tau0Text.setEnabled(false);
                _qText.setEnabled(false);
                _numberAntText.setEnabled(false);
                _skIterationButton.setEnabled(false);
                _skIterationText.setEnabled(false);
                _skThresholdButton.setEnabled(false);
                _skThresholdText.setEnabled(false);
                _skSolutionButton.setEnabled(false);
                _skSolutionBrowseButton.setEnabled(false);
                _skSolutionBrowseText.setEnabled(false);
                
            }
            
        });
        
    }
    


    /**
     * aktualisiert bei eintragen von Werten den Zustand des Startbuttons
     */
    public void refreshStartButton() {
        if ((Boolean) _alphaText.getData() && (Boolean) _betaText.getData() && (Boolean) _rhoText.getData() && (Boolean) _tau0Text.getData() && (Boolean) _qText.getData()
                && (Boolean) _numberAntText.getData() && _controller.coordinatesSet()) {
            if (_skIterationButton.getSelection() && (Boolean) _skIterationText.getData()) {
                _startButton.setEnabled(true);
            }
            else if (_skSolutionButton.getSelection() && (Boolean) _skSolutionBrowseText.getData()) {
                _startButton.setEnabled(true);
            }
            else if (_skThresholdButton.getSelection() && (Boolean) _skThresholdText.getData()) {
                _startButton.setEnabled(true);
            }
            else {
                _startButton.setEnabled(false);
            }
        }
        else {
            _startButton.setEnabled(false);
        }
    }
    


    // SETTER zum setzen der Werte der Textboxen
    
    public void setAlphaTextValue(String value) {
        _alphaText.setText(value);
    }
    


    public void setBetaTextValue(String value) {
        _betaText.setText(value);
    }
    


    public void setRhoTextValue(String value) {
        _rhoText.setText(value);
    }
    


    public void setTau0TextValue(String value) {
        _tau0Text.setText(value);
    }
    


    public void setqTextValue(String value) {
        _qText.setText(value);
    }
    


    public void setSkIterationCount(int count) {
        _skIterationText.setText("" + count);
        
    }
    


    public void setNumberAntCount(int count) {
        _numberAntText.setText("" + count);
        
    }
    


    public void setSkIterationButtonEnabled() {
        _skIterationText.setEnabled(true);
        
        // Andere Buttons mssen auf false gesetzt werden
        _skIterationButton.setSelection(true);
        _skSolutionButton.setSelection(false);
        _skThresholdButton.setSelection(false);
        
        _skThresholdText.setBackground(_gray);
        _skThresholdText.setEnabled(false);
        _skSolutionBrowseButton.setBackground(_gray);
        _skSolutionBrowseButton.setEnabled(false);
        _skSolutionBrowseText.setBackground(_gray);
        _skSolutionBrowseText.setEnabled(false);
    }
    


    /**
     * gibt zurck ob der Algoritmus luft
     * 
     * @return
     */
    public boolean isRunning() {
        if (_startButton.getText() == "STOP") {
            return true;
        }
        return false;
    }
    


    /**
     * beendet den thread
     */
    @SuppressWarnings("deprecation")
    public void stop() {
        if (thread != null) {
            thread.stop();
            thread = null;
        }
    }
    
}
