/**
 * 
 */
package de.hwrBerlin.ameisenproblem.view;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import de.hwrBerlin.ameisenproblem.controller.MainController;
import de.hwrBerlin.ameisenproblem.databeans.Coordinate;

/**
 * @author Nico Smeenk
 * 
 */
public class CoordinateDialog extends Composite {
    
    /**
     * @param pParent
     * @param pStyle
     * @param pController
     */
    public CoordinateDialog(Composite pParent, int pStyle, final MainController pController, double pX, double pY) {
        super(pParent, pStyle);
        setLayout(new GridLayout(2, true));
        
        GridData data = new GridData(SWT.FILL, SWT.FILL, false, false);
        
        new Label(this, SWT.NONE).setText("Name: ");
        
        final Text nameText = new Text(this, SWT.BORDER);
        nameText.setLayoutData(data);
        nameText.setData(false);
        
        new Label(this, SWT.NONE).setText("x Koordinate: ");
        
        final Text xText = new Text(this, SWT.BORDER);
        xText.setLayoutData(data);
        xText.setData(false);
        
        new Label(this, SWT.NONE).setText("y Koordinate: ");
        
        final Text yText = new Text(this, SWT.BORDER);
        yText.setLayoutData(data);
        yText.setData(false);
        
        final Button okButton = new Button(this, SWT.PUSH);
        okButton.setText("OK");
        okButton.setLayoutData(data);
        okButton.setEnabled(false);
        
        Button cancelButton = new Button(this, SWT.PUSH);
        cancelButton.setText("Abbrechen");
        cancelButton.setLayoutData(data);
        
        final Color valid = new Color(getDisplay(), Constants.WHITE_COLOR);
        final Color notValid = new Color(getDisplay(), Constants.RED_COLOR);
        
        
        // nur richtige Eingaben
        ModifyListener coordinateModifyListener = new ModifyListener() {
            
            @Override
            public void modifyText(ModifyEvent pE) {
                Text text = (Text) pE.widget;
                if (text.getText().matches("([0-9][0-9]*\\.[0-9]*|[0-9][0-9]*)")){
                    text.setBackground(valid);
                    text.setData(true);
                }
                else {
                    text.setBackground(notValid);
                    text.setData(false);
                }
            }
        };
        
        xText.addModifyListener(coordinateModifyListener);
        yText.addModifyListener(coordinateModifyListener);

        nameText.addModifyListener(new ModifyListener() {
            
            @Override
            public void modifyText(ModifyEvent pE) {
                Text text = (Text) pE.widget;
                if (pController.isCityNameInUse(text.getText())){
                    text.setBackground(notValid);
                    text.setData(false);
                }
                else {
                    text.setBackground(valid);
                    text.setData(true);
                }
            }
        });
        
        // Listener zum ndern des Buttons
        ModifyListener refreshButtonListener = new ModifyListener() {
            
            @Override
            public void modifyText(ModifyEvent pE) {
                if ((Boolean) nameText.getData() && (Boolean) xText.getData() && (Boolean) yText.getData()){
                    okButton.setEnabled(true);
                }
                else {
                    okButton.setEnabled(false);
                }
            }
        };
        
        nameText.addModifyListener(refreshButtonListener);
        xText.addModifyListener(refreshButtonListener);
        yText.addModifyListener(refreshButtonListener);

        
        // Abbrechen
        cancelButton.addSelectionListener(new SelectionAdapter() {
            /* (non-Javadoc)
             * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
             */
            @Override
            public void widgetSelected(SelectionEvent pE) {
                getShell().close();
            }
        });
        
        //ok
        okButton.addSelectionListener(new SelectionAdapter() {
            /* (non-Javadoc)
             * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
             */
            @Override
            public void widgetSelected(SelectionEvent pE) {
                Coordinate newCoordinate = new Coordinate();
                newCoordinate.setName(nameText.getText());
                newCoordinate.setxValue(Double.parseDouble(xText.getText()));
                newCoordinate.setyValue(Double.parseDouble(yText.getText()));
                
                pController.addCoordinate(newCoordinate);
                getShell().close();
            }
        });
        
        
        nameText.setText("new");
        xText.setText("" + pX);
        yText.setText("" + pY);
    }
    
}
