/**
 * 
 */
package de.hwrBerlin.ameisenproblem.view;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

import de.hwrBerlin.ameisenproblem.controller.MainController;
import de.hwrBerlin.ameisenproblem.databeans.Coordinate;

/**
 * @author Nico Smeenk
 * 
 */
public class GraphView extends ScrolledComposite {
    
    /** momantane Liste der Punkte */
    private List<Coordinate> _currentPoints = new ArrayList<Coordinate>();
    /** bester Weg (der angezeigt werden soll) in einer liste */
    private List<Coordinate> _way = new ArrayList<Coordinate>();
    /** bester Weg einer iteration (der angezeigt werden soll) in einer liste */
    private List<Coordinate> _iterationWay = new ArrayList<Coordinate>();
    /** Maximale x-gre */
    private int _maxX;
    /** Maximale y-gre */
    private int _maxY;
    /** zoomfaktor */
    private double _divisor = 1.0;
    /** Zeichencomposite */
    private Composite _graphComposite;
    /** Controller */
    private MainController _controller;
    
    private static final int BORDER = 5;
    
    

    /**
     * @param pParent
     * @param pStyle
     * @param pController
     */
    public GraphView(Composite pParent, int pStyle, MainController pController) {
        super(pParent, pStyle);
        
        _controller = pController;
        
        _graphComposite = new Composite(this, SWT.NONE);
        
        final Color cityColor = new Color(getDisplay(), 0, 0, 255);
        final Color bestWayColor = new Color(getDisplay(), 255, 0, 0);
        final Color bestIterationWayColor = new Color(getDisplay(), 200, 200, 200);
        
        // Grafik zeichnen
        _graphComposite.addPaintListener(new PaintListener() {
            
            @Override
            public void paintControl(PaintEvent pE) {
                
                // besten weg einer Iteration zeichnen
                pE.gc.setForeground(bestIterationWayColor);
                paintWay(cityColor, pE, _iterationWay);
                
                // besten weg zeichnen
                pE.gc.setForeground(bestWayColor);
                paintWay(cityColor, pE, _way);
            }
            


            /**
             * @param cityColor
             * @param pE
             * @param pWay
             */
            private void paintWay(final Color cityColor, PaintEvent pE, List<Coordinate> pWay) {
                if (pWay.size() > 1) {
                    
                    for (int coordinateIndex = 0; coordinateIndex < pWay.size() - 1; coordinateIndex++) {
                        int xStart = (int) (pWay.get(coordinateIndex).getxValue() / _divisor + BORDER);
                        int yStart = (int) (pWay.get(coordinateIndex).getyValue() / _divisor + BORDER);
                        int xEnd = (int) (pWay.get(coordinateIndex + 1).getxValue() / _divisor + BORDER);
                        int yEnd = (int) (pWay.get(coordinateIndex + 1).getyValue() / _divisor + BORDER);
                        pE.gc.drawLine(xStart, yStart, xEnd, yEnd);
                    }
                    int xStart = (int) (pWay.get(pWay.size() - 1).getxValue() / _divisor + BORDER);
                    int yStart = (int) (pWay.get(pWay.size() - 1).getyValue() / _divisor + BORDER);
                    int xEnd = (int) (pWay.get(0).getxValue() / _divisor + BORDER);
                    int yEnd = (int) (pWay.get(0).getyValue() / _divisor + BORDER);
                    pE.gc.drawLine(xStart, yStart, xEnd, yEnd);
                }
                pE.gc.setBackground(cityColor);
                for (Coordinate coordinate : _currentPoints) {
                    pE.gc.fillOval((int) (coordinate.getxValue() / _divisor) - 3 + BORDER, (int) (coordinate.getyValue() / _divisor) - 3 + BORDER, 7, 7);
                }
            }
        });
        
        // Neuer Punkt bei Klick
        _graphComposite.addMouseListener(new MouseAdapter() {
            /*
             * (non-Javadoc)
             * 
             * @see org.eclipse.swt.events.MouseAdapter#mouseDown(org.eclipse.swt.events.MouseEvent)
             */
            @Override
            public void mouseDown(MouseEvent pE) {
                if (!_controller.isRunning()) {
                    Shell shell = new Shell(getDisplay(), SWT.CLOSE);
                    shell.setImages(Constants.ICON_ARRAY);
                    shell.setText(Constants.NEW_COORDINATE_TITLE);
                    shell.setLayout(new FillLayout());
                    
                    new CoordinateDialog(shell, SWT.NONE, _controller, pE.x * _divisor, pE.y * _divisor);
                    
                    shell.pack();
                    shell.open();
                }
            }
        });
        setContent(_graphComposite);
    }
    


    /**
     * Zeichnet den besten weg ein
     * 
     * @param pCoordinates
     */
    public void paintBestWay(List<Coordinate> pCoordinates) {
        _way = pCoordinates;
        _graphComposite.redraw();
    }
    


    /**
     * Zeichnet den besten weg einer Iteration ein
     * 
     * @param pCoordinates
     */
    public void paintBestIterationWay(List<Coordinate> pCoordinates) {
        _iterationWay = pCoordinates;
        _graphComposite.redraw();
    }
    


    /**
     * @param pShortestWay
     * @param pShortestIterationWay
     */
    public void paintWays(List<Coordinate> pShortestWay, List<Coordinate> pShortestIterationWay) {
        _way = pShortestWay;
        _iterationWay = pShortestIterationWay;
        _graphComposite.redraw();
    }
    


    /**
     * Koordinaten zeichnen
     * 
     * @param pCoordinates
     *            Koordinaten
     */
    public void paintCoordinates(List<Coordinate> pCoordinates) {
        _way = new ArrayList<Coordinate>();
        _iterationWay = new ArrayList<Coordinate>();
        _currentPoints = pCoordinates;
        
        // maximale werte
        _maxX = 0;
        _maxY = 0;
        for (Coordinate coordinate : _currentPoints) {
            if (coordinate.getxValue() > _maxX) {
                _maxX = (int) coordinate.getxValue();
            }
            if (coordinate.getyValue() > _maxY) {
                _maxY = (int) coordinate.getyValue();
            }
        }
        
        _graphComposite.setSize((int) (_maxX / _divisor) + (2 * BORDER), (int) (_maxY / _divisor) + (2 * BORDER));
        layout();
        _graphComposite.redraw();
    }
    


    /**
     * Zoomt den Grafen
     * 
     * @param pSelection
     */
    public void zoom(int pSelection) {
        _divisor = (double) (100 - pSelection) / (double) 50;
        if (_divisor == 0) {
            _divisor = 0.01;
        }
        if (pSelection < 50) {
            _divisor = _divisor * _divisor * _divisor * _divisor;
        }
        List<Coordinate> way = _way;
        paintCoordinates(_currentPoints);
        paintBestWay(way);
    }
    
}
