/**
 * 
 */
package de.hwrBerlin.ameisenproblem.view;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Shell;

import de.hwrBerlin.ameisenproblem.controller.MainController;
import de.hwrBerlin.ameisenproblem.databeans.Coordinate;
import de.hwrBerlin.ameisenproblem.databeans.TSPScenario;
import de.hwrBerlin.ameisenproblem.exceptionhandling.ShowableException;

/**
 * @author Nico Smeenk
 * 
 *         Kmmert sich um die Grafische Darstellung
 */
public class MainView {
    
    /** Controller fr die Logik */
    private MainController _controller;
    
    /** Grafische Anzeige */
    private GraphView _graphView;
    
    /** Ergebnisview */
    private ResultView _resultView;
    
    /** Parameter etc eingeben */
    private ConfigurationView _configurationView;
    
    /** Display */
    private Display _display;
    
    

    /**
     * Konstruktor fr das Hauptfenster
     * 
     * @param pMainController
     */
    public MainView(MainController pMainController) {
        
        _controller = pMainController;
        _controller.setView(this);
        
        // sicherheitshalbe nullcheck
        if (_controller == null) {
            throw new NullPointerException();
        }
        
        // View erstellen
        _display = new Display();
        Shell shell = new Shell(_display);
        shell.setImages(Constants.ICON_ARRAY);
        shell.setText(Constants.MAIN_TITLE);
        shell.setLayout(new GridLayout(2, false));
        
        createMenu(shell);
        
        Composite graphViewComposite = new Composite(shell, SWT.None);
        
        graphViewComposite.setLayout(new GridLayout(2, false));
        graphViewComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        
        _graphView = new GraphView(graphViewComposite, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL, _controller);
        _graphView.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        
        // Grafen zoomen
        Composite zoomComposite = new Composite(graphViewComposite, SWT.NONE);
        zoomComposite.setLayout(new GridLayout(1, true));
        zoomComposite.setLayoutData(new GridData(SWT.CENTER, SWT.FILL, false, true));
        
        Button zoomOutButton = new Button(zoomComposite, SWT.PUSH);
        zoomOutButton.setImage(new Image(_display, Constants.ZOOM_OUT_IMAGE));
        zoomOutButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        
        final Scale scaleBar = new Scale(zoomComposite, SWT.VERTICAL);
        scaleBar.setLayoutData(new GridData(SWT.CENTER, SWT.FILL, false, true));
        scaleBar.setSelection(50);
        
        Button zoomInButton = new Button(zoomComposite, SWT.PUSH);
        zoomInButton.setImage(new Image(_display, Constants.ZOOM_IN_IMAGE));
        zoomInButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        
        zoomOutButton.addSelectionListener(new SelectionAdapter() {
            /*
             * (non-Javadoc)
             * 
             * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
             */
            @Override
            public void widgetSelected(SelectionEvent pE) {
                if (scaleBar.getSelection() > 0) {
                    scaleBar.setSelection(scaleBar.getSelection() - 1);
                    _graphView.zoom(scaleBar.getSelection());
                }
            }
        });
        zoomInButton.addSelectionListener(new SelectionAdapter() {
            /*
             * (non-Javadoc)
             * 
             * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
             */
            @Override
            public void widgetSelected(SelectionEvent pE) {
                if (scaleBar.getSelection() < 100) {
                    scaleBar.setSelection(scaleBar.getSelection() + 1);
                    _graphView.zoom(scaleBar.getSelection());
                }
            }
        });
        
        scaleBar.addSelectionListener(new SelectionAdapter() {
            /*
             * (non-Javadoc)
             * 
             * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
             */
            @Override
            public void widgetSelected(SelectionEvent pE) {
                _graphView.zoom(scaleBar.getSelection());
            }
        });
        Composite dataViewComposite = new Composite(shell, SWT.None);
        dataViewComposite.setLayout(new GridLayout(1, false));
        GridData gridData = new GridData(SWT.CENTER, SWT.TOP, false, false);
        gridData.horizontalSpan = 0;
        dataViewComposite.setLayoutData(gridData);
        
        _configurationView = new ConfigurationView(dataViewComposite, SWT.BORDER, _controller);
        _resultView = new ResultView(dataViewComposite, SWT.BORDER, _controller);
        
        shell.setMaximized(true);
        shell.open();
        
        while (!shell.isDisposed()) {
            if (!_display.readAndDispatch()) {
                _display.sleep();
            }
        }
        _controller.setIsRunning(false);
        _configurationView.stop();
        _display.dispose();
        
    }
    


    /**
     * Erzeugt die Menleiste
     * 
     * @param shell
     */
    private void createMenu(final Shell shell) {
        // Erstellung des Mens
        Menu menuBar = new Menu(shell, SWT.BAR);
        MenuItem fileMenuHeader = new MenuItem(menuBar, SWT.CASCADE);
        fileMenuHeader.setText("&Datei");
        
        Menu fileMenu = new Menu(shell, SWT.DROP_DOWN);
        fileMenuHeader.setMenu(fileMenu);
        
        MenuItem fileOpenItem = new MenuItem(fileMenu, SWT.PUSH);
        fileOpenItem.setText("&ffnen");
        
        MenuItem fileSaveItem = new MenuItem(fileMenu, SWT.PUSH);
        fileSaveItem.setText("TSP &Speichern");
        
        MenuItem fileSaveResultItem = new MenuItem(fileMenu, SWT.PUSH);
        fileSaveResultItem.setText("&Ergebnis Speichern");
        
        MenuItem randomValuesItem = new MenuItem(fileMenu, SWT.PUSH);
        randomValuesItem.setText("&Zufallsparameter");
        
        MenuItem randomSzenarioItem = new MenuItem(fileMenu, SWT.PUSH);
        randomSzenarioItem.setText("&Zufallsszenario");
        
        MenuItem helpMenuHeader = new MenuItem(menuBar, SWT.CASCADE);
        helpMenuHeader.setText("&Hilfe");
        
        Menu helpMenu = new Menu(shell, SWT.DROP_DOWN);
        helpMenuHeader.setMenu(helpMenu);
        
        MenuItem aboutItem = new MenuItem(helpMenu, SWT.PUSH);
        aboutItem.setText("&ber...");
        
        MenuItem handbookItem = new MenuItem(helpMenu, SWT.PUSH);
        handbookItem.setText("&Handbuch");
        
        // men hinzufgen
        shell.setMenuBar(menuBar);
        
        // Verwaltung des Mens
        // Datei ffnen
        fileOpenItem.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent pE) {
                System.out.println(getClass() + " Men Datei ffnen");
                
                if (_controller.isRunning()) {
                    MessageBox messageBox = new MessageBox(shell, SWT.ICON_ERROR);
                    messageBox.setText(Constants.ERROR_TITLE_ON_OPEN_WHILE_RUNNING);
                    messageBox.setMessage(Constants.ERROR_TEXT_ON_OPEN_WHILE_RUNNING);
                    messageBox.open();
                }
                else {
                    
                    FileDialog fileOpenDialog = new FileDialog(shell, SWT.OPEN);
                    fileOpenDialog.setFilterExtensions(Constants.TSP_FILE_EXTENSION);
                    String fileOpenName = fileOpenDialog.open();
                    System.out.println(getClass() + " Pfad der zu ffnenden Datei: " + fileOpenName);
                    if (fileOpenName != null) {
                        try {
                            _controller.openTsp(fileOpenName);
                        }
                        catch (ShowableException e) {
                            System.out.println(getClass() + " Fehler beim Einlesen der Datei:");
                            e.printStackTrace();
                            MessageBox errorBox = new MessageBox(shell, SWT.ICON_ERROR);
                            errorBox.setText(e.getTitle());
                            errorBox.setMessage(e.getMessage());
                            errorBox.open();
                        }
                    }
                }
            }
        });
        // Datei speichern
        fileSaveItem.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent pE) {
                System.out.println(getClass() + " Men Datei speichern ");
                if (_controller.getCoordinates().size() <= 0) {
                    MessageBox messageBox = new MessageBox(shell, SWT.ICON_ERROR);
                    messageBox.setText(Constants.ERROR_TITLE_SAVE_NO_POINTS);
                    messageBox.setMessage(Constants.ERROR_TEXT_SAVE_NO_POINTS);
                    messageBox.open();
                }
                else {
                    FileDialog fileSaveDialog = new FileDialog(shell, SWT.SAVE);
                    fileSaveDialog.setFilterExtensions(Constants.TSP_FILE_EXTENSION);
                    String fileSaveName = fileSaveDialog.open();
                    System.out.println(getClass() + " Speichern der Datei unter: " + fileSaveName);
                    if (fileSaveName != null) {
                        _controller.save(fileSaveName);
                    }
                }
            }
        });
        // Ergebnis speichern
        fileSaveResultItem.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent pE) {
                System.out.println(getClass() + " Men Ergebnis speichern");
                if (_controller.getShortestWay() != null) {
                    FileDialog fileSaveResultDialog = new FileDialog(shell, SWT.SAVE);
                    fileSaveResultDialog.setFilterExtensions(Constants.TOUR_FILE_EXTENSION);
                    String fileSaveResultName = fileSaveResultDialog.open();
                    System.out.println(getClass() + " Speichern des Ergebnisses unter: " + fileSaveResultName);
                    if (fileSaveResultName != null) {
                        _controller.saveResult(fileSaveResultName);
                    }
                }
                else {
                    MessageBox messageBox = new MessageBox(shell, SWT.ICON_ERROR);
                    messageBox.setText(Constants.ERROR_TITLE_SAVE_RESULT_NO_RESULT);
                    messageBox.setMessage(Constants.ERROR_TEXT_SAVE_RESULT_NO_RESULT);
                    messageBox.open();                    
                }
            }
        });
        // ZufallsWerte erstellen
        randomValuesItem.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent pE) {
                System.out.println(getClass() + " Men ZufallsWerte erstellen");
                if (_controller.isRunning()) {
                    MessageBox messageBox = new MessageBox(shell, SWT.ICON_ERROR);
                    messageBox.setText(Constants.ERROR_TITLE_ON_RANDOM_WHILE_RUNNING);
                    messageBox.setMessage(Constants.ERROR_TEXT_ON_RANDOM_WHILE_RUNNING);
                    messageBox.open();
                }
                else {
                    // XXX hier kann ein Dialog fr die Anzahl der Stdte hinzukommen
                    _controller.createRandomValues();
                }
            }
        });
        // Zufallsszenario erstellen
        randomSzenarioItem.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent pE) {
                System.out.println(getClass() + " Men Zufallsszenario erstellen");
                if (_controller.isRunning()) {
                    MessageBox messageBox = new MessageBox(shell, SWT.ICON_ERROR);
                    messageBox.setText(Constants.ERROR_TITLE_ON_RANDOM_WHILE_RUNNING);
                    messageBox.setMessage(Constants.ERROR_TEXT_ON_RANDOM_WHILE_RUNNING);
                    messageBox.open();
                }
                else {
                    // XXX hier kann ein Dialog fr die Anzahl der Stdte hinzukommen
                    _controller.createRandomSzenario();
                }
            }
        });
        // Versions/Informationen
        aboutItem.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent pE) {
                System.out.println(getClass() + " About Dialog aufrufen");
                
                Shell aboutShell = new Shell(shell.getDisplay(), SWT.CLOSE);
                aboutShell.setImages(Constants.ICON_ARRAY);
                aboutShell.setText(Constants.ABOUT_TITLE);
                aboutShell.setLayout(new FillLayout());
                
                new AboutDialog(aboutShell, SWT.NONE);
                
                aboutShell.pack();
                aboutShell.open();
            }
        });
        // Handbuch
        handbookItem.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent pE) {
                System.out.println(getClass() + " Handbuch ffnen");
                
                Shell aboutShell = new Shell(shell.getDisplay());
                aboutShell.setImages(Constants.ICON_ARRAY);
                aboutShell.setText(Constants.HANDBOOK_TITLE);
                aboutShell.setLayout(new FillLayout());
                
                new HandbookDialog(aboutShell, SWT.NONE);
                
                aboutShell.setSize(1000, 600);
                aboutShell.open();
            }
        });
    }
    


    /**
     * Koordinaten Zeichnen
     * 
     * @param pCoordinates
     */
    public void paintCoordinates(List<Coordinate> pCoordinates) {
        _graphView.paintCoordinates(pCoordinates);
        _resultView.setCoordinateCount(pCoordinates.size());
        
        _configurationView.refreshStartButton();
    }
    


    /**
     * Zufallsszenario einsetzen
     * 
     * @param scenario
     * @param withSKIeration
     */
    public void setRandomScenario(TSPScenario scenario, boolean withSKIeration) {
        
        _configurationView.setAlphaTextValue("" + scenario.getAlpha());
        _configurationView.setBetaTextValue("" + scenario.getBeta());
        _configurationView.setNumberAntCount(scenario.getNumberAnt());
        _configurationView.setqTextValue("" + scenario.getQ());
        _configurationView.setRhoTextValue("" + scenario.getRho());
        
        _configurationView.setTau0TextValue("" + scenario.getTau0());
        
        // Sollen auch zufllige StoppKriteriumswerte generiert werden?
        if (withSKIeration) {
            _configurationView.setSkIterationCount(scenario.getSkIteration());
            _configurationView.setSkIterationButtonEnabled();
        }
        
    }
    


    /**
     * 
     * Kmmert sich um alle Daten die eingetragen werden sollen
     * 
     * @param pProgress
     *            Fortschritt in Prozent
     * @param pTime
     *            Zeit in ms
     * @param pShortestWay
     *            Krzester Weg
     * @param pShortestIterationWay
     *            Krzester weg der Iteration
     * @param pAverage
     *            durchschnittlicher Weg
     * @param pIterationAverage
     *            durchschnittlicher Weg der Iteration
     */
    public void showIteration(final int pProgress, final long pTime, final List<Coordinate> pShortestWay, final List<Coordinate> pShortestIterationWay, final double pAverage,
            final Double pIterationAverage) {
        if (_display.isDisposed()) {
            return;
        }
        _display.asyncExec(new Runnable() {
            @Override
            public void run() {
                _resultView.showIteration(pProgress, pTime, pShortestWay, pShortestIterationWay, pAverage, pIterationAverage);
                _graphView.paintWays(pShortestWay, pShortestIterationWay);
            }
        });
    }
}
