/**
 * 
 */
package de.hwrBerlin.ameisenproblem.view;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Text;

import de.hwrBerlin.ameisenproblem.controller.MainController;
import de.hwrBerlin.ameisenproblem.databeans.Coordinate;

/**
 * 
 * @author Christian Gebauer, Sebastian Groe
 * 
 *         Anzeige der Ausgaben fr das TSP
 * 
 */
public class ResultView extends Composite {
    
    /** Controller fr die Logik */
    // private MainController _controller;
    
    private Text _averageTourIterationOut;
    private Text _averageTourGlobalOut;
    private Text _timeLeftOut;
    private Text _numberCityOut;
    private Text _bestTourIterationOut;
    private Text _bestTourGlobalOut;
    private ProgressBar _progressBar;
    
    

    /**
     * @param pParent
     * @param pStyle
     */
    public ResultView(Composite pParent, int pStyle, MainController pMainController) {
        super(pParent, pStyle);
        
        // _controller = pMainController;
        
        setLayout(new RowLayout(SWT.VERTICAL));
        
        RowLayout rowLayout = new RowLayout();
        rowLayout.center = true;
        
        // RowData defaultRowData = new RowData(16, 16);
        
        // AusgabeFelderder Parameter
        // Anzahl Stdte
        Label resultTitleLabel = new Label(this, SWT.NONE);
        resultTitleLabel.setText("Ausgabe");
        resultTitleLabel.setFont(new Font(pParent.getDisplay(), "Arial", 16, SWT.BOLD));
        
        Composite numberCityComposite = new Composite(this, SWT.NONE);
        numberCityComposite.setLayout(rowLayout);
        
        Label numberCityLabel = new Label(numberCityComposite, SWT.NONE);
        numberCityLabel.setText(Constants.NUMBER_CITY_TXT);
        
        _numberCityOut = new Text(numberCityComposite, SWT.READ_ONLY | SWT.BORDER);
        _numberCityOut.setText("--------------");
        
        Label lineLabel = new Label(this, SWT.SEPARATOR | SWT.HORIZONTAL | SWT.CENTER);
        lineLabel.setLayoutData(new RowData(320, 10));
        
        // Lnge zur zeit beste Tour
        Label bestTourLabel = new Label(this, SWT.NONE);
        bestTourLabel.setText(Constants.BEST_TOUR_TXT);
        
        Composite bestTourComposite = new Composite(this, SWT.NONE);
        bestTourComposite.setLayout(rowLayout);
        
        Label bestTourIterationLabel = new Label(bestTourComposite, SWT.NONE);
        bestTourIterationLabel.setText(Constants.ITERATION_TXT);
        
        _bestTourIterationOut = new Text(bestTourComposite, SWT.READ_ONLY | SWT.BORDER);
        _bestTourIterationOut.setText("--------------");
        
        Label bestTourGlobalLabel = new Label(bestTourComposite, SWT.NONE);
        bestTourGlobalLabel.setText(Constants.GLOBAL_TXT);
        
        _bestTourGlobalOut = new Text(bestTourComposite, SWT.READ_ONLY | SWT.BORDER);
        _bestTourGlobalOut.setText("--------------");
        
        Label lineLabel2 = new Label(this, SWT.SEPARATOR | SWT.HORIZONTAL | SWT.CENTER);
        lineLabel2.setLayoutData(new RowData(320, 10));
        
        // Lnge durchschnittliche Tour
        Label averageTourLabel = new Label(this, SWT.NONE);
        averageTourLabel.setText(Constants.AVERAGE_TOUR_TXT);
        
        Composite averageTourComposite = new Composite(this, SWT.NONE);
        averageTourComposite.setLayout(rowLayout);
        
        Label averageTourIterationLabel = new Label(averageTourComposite, SWT.NONE);
        averageTourIterationLabel.setText(Constants.ITERATION_TXT);
        
        _averageTourIterationOut = new Text(averageTourComposite, SWT.READ_ONLY | SWT.BORDER);
        _averageTourIterationOut.setText("--------------");
        
        Label averageTourGlobalLabel = new Label(averageTourComposite, SWT.NONE);
        averageTourGlobalLabel.setText(Constants.GLOBAL_TXT);
        
        _averageTourGlobalOut = new Text(averageTourComposite, SWT.READ_ONLY | SWT.BORDER);
        _averageTourGlobalOut.setText("--------------");
        
        Label lineLabel3 = new Label(this, SWT.SEPARATOR | SWT.HORIZONTAL | SWT.CENTER);
        lineLabel3.setLayoutData(new RowData(320, 10));
        
        Composite timeLeftComposite = new Composite(this, SWT.NONE);
        timeLeftComposite.setLayout(rowLayout);
        
        Label timeLeftLabel = new Label(timeLeftComposite, SWT.NONE);
        timeLeftLabel.setText(Constants.TIME_LEFT_TXT);
        
        _timeLeftOut = new Text(timeLeftComposite, SWT.READ_ONLY | SWT.BORDER);
        _timeLeftOut.setText("--------------");
        
        Label lineLabel4 = new Label(this, SWT.SEPARATOR | SWT.HORIZONTAL | SWT.CENTER);
        lineLabel4.setLayoutData(new RowData(320, 10));
        
        // Statusbar
        Composite progressBarComposite = new Composite(this, SWT.NONE);
        
        _progressBar = new ProgressBar(progressBarComposite, SWT.SMOOTH);
        _progressBar.setBounds(0, 0, 315, 30);
        // progressBar.setSelection(30);
    }
    


    /**
     * @param pSize
     */
    public void setCoordinateCount(int pSize) {
        _numberCityOut.setText(pSize + "");
    }
    


    /**
     * 
     * Kmmert sich um alle Daten die eingetragen werden sollen
     * 
     * @param pProgress
     *            Fortschritt in Prozent
     * @param pTime
     *            Zeit in ms
     * @param pShortestWay
     *            Krzester Weg
     * @param pShortestIterationWay
     *            Krzester weg der Iteration
     * @param pAverage
     *            durchschnittlicher Weg
     * @param pIterationAverage
     *            durchschnittlicher Weg der Iteration
     */
    public void showIteration(int pProgress, long pTime, List<Coordinate> pShortestWay, List<Coordinate> pShortestIterationWay, double pAverage, Double pIterationAverage) {
        _progressBar.setSelection(pProgress);
        _timeLeftOut.setText("" + pTime);
        _bestTourGlobalOut.setText("" + MainController.calculateWayLength(pShortestWay));
        _bestTourIterationOut.setText("" + MainController.calculateWayLength(pShortestIterationWay));
        _averageTourGlobalOut.setText("" + pAverage);
        _averageTourIterationOut.setText("" + pIterationAverage);
    }
    
}
